/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.stack.impl;

import org.tinygroup.commons.exceptions.FullException;
import org.tinygroup.stack.Stack;
import org.tinygroup.stack.StackMonitor;

/**
 * 采用了数组的堆栈 User: luoguo Date: 11-4-2 Time: 上午8:56 To change this template use
 * File | Settings | File Templates.
 */
public class StackArray<E> implements Stack<E>, StackMonitor {

	private String name = null;
	private int maxSize;
	private E[] array;
	private int size = 0;

	public void setName(String name) {
		this.name = name;
	}

	public StackArray() {
		this(DEFAULT_STACK_SIZE);
	}

	@SuppressWarnings("unchecked")
	public StackArray(int maxSize) {
		this.maxSize = maxSize;
		array = (E[]) new Object[maxSize];
	}

	public boolean isEmpty() {
		return size == 0;
	}

	public E peek() {
		if (size == 0) {
			return null;
		}
		return array[size - 1];
	}

	public E pop() {
		synchronized (array) {
			if (size == 0) {
				return null;
			}
			size--;
			E temp = array[size];
			array[size] = null;
			return temp;
		}
	}

	public E push(E item) {
		synchronized (array) {
			if (size >= maxSize) {
				throw new FullException();
			}
			array[size++] = item;
			return item;
		}
	}

	public String getName() {
		if (name == null) {
			return this.getClass().getSimpleName();
		} else {
			return name;
		}
	}

	public int size() {
		return size;
	}

	public int getUsingLength() {
		return size;
	}

	public int getIdleLength() {
		return maxSize - size;
	}

	public int getMaxSize() {
		return maxSize;
	}

	@SuppressWarnings("unchecked")
	public void clear() {
		array = (E[]) new Object[maxSize];
		size = 0;

	}
}
