/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.stack.impl;

import org.tinygroup.commons.exceptions.FullException;
import org.tinygroup.linklist.LinkList;
import org.tinygroup.linklist.impl.LinkListImpl;
import org.tinygroup.stack.Stack;

/**
 * 采用了单向链表的堆栈 User: luoguo Date: 11-4-2 Time: 上午8:56 To change this template use
 * File | Settings | File Templates.
 */
public class StackLinkList<E> implements Stack<E> {
	private String name = null;
	private int maxSize = DEFAULT_STACK_SIZE;
	private LinkList<E> stackList = new LinkListImpl<E>();

	public void setName(String name) {
		this.name = name;
	}

	public StackLinkList() {
		this(DEFAULT_STACK_SIZE);
	}

	public StackLinkList(int maxSize) {
		this.maxSize = maxSize;
	}

	public boolean isEmpty() {
		return stackList.size() == 0;
	}

	public E peek() {
		return stackList.getFirst();
	}

	public E pop() {
		return stackList.removeFirst();
	}

	public E push(E item) {
		if (maxSize != 0 && maxSize <= stackList.size()) {
			throw new FullException("Stack is full.");
		}
		return stackList.addFirst(item);
	}

	public String getName() {
		if (name == null) {
			return this.getClass().getSimpleName();
		} else {
			return name;
		}
	}

	public int getMaxSize() {
		return maxSize;
	}

	public int getUsingLength() {
		return this.stackList.size();
	}

	public int getIdleLength() {
		return maxSize - getUsingLength();
	}

	public void clear() {
		stackList.clear();
	}

	public int size() {
		return stackList.size();
	}
}
